try:
    import poser
except ImportError:
    from PoserLibs import POSER_FAKE as poser

import WXtools

reload(WXtools)
from WXtools import *

SCENE = poser.Scene()


class Deformer(object):
    __slots__ = ("mag",  # the magnet -
                 "base",  # its base -
                 "zones",  # and the zones.
                 )

    def __init__(self, mag, base, zones):
        self.mag = mag
        self.base = base
        self.zones = zones


def getMagnetSet(ac_or_name):
    if isinstance(ac_or_name, basestring):
        ac = SCENE.actor(ac_or_name)
    else:
        ac = ac_or_name

    base = mag = zone = None
    if ac.IsBase():
        base = ac
        mag = base.Children()[0]
    elif ac.IsDeformer():
        mag = ac
        base = mag.Parent()
    elif ac.IsZone():
        zone = ac
        common_name = zone.Name()[:-1]
        for p in ac.Parent().Children():
            if p.Name().startwith(common_name):
                if p.Name()[-1] == "B":
                    base = p
                elif p.Name()[-1] == "M":
                    mag = p

    return mag, base


def collectMagnets(base=SCENE):
    ar = list()
    for ac in base.Actors():
        if ac.IsDeformer():
            ar.append(ac.InternalName())


def CreateMagnet(name, parent=None):
    for ac in SCENE.Actors():  # type: poser.ActorType
        if ac.IsDeformer() and ac.Name() == name:
            return getMagnetSet(ac)

    ac = SCENE.CurrentActor()

    if isinstance(parent, poser.ActorType):
        SCENE.SelectActor(parent)
    else:
        parent = ac

    mag = SCENE.CreateMagnet()  # type: poser.ActorType
    oldname = mag.Name()
    zone = mag.Zones()[0]
    base = mag.Parent()

    mag.SetName(name + "_Mag")
    base.SetName(name + "_Base")
    if len(mag.Zones()) > 1:
        for idx, z in enumerate(mag.Zones()):
            zone.SetName(name + "_(%d)Zone" % idx)
    else:
        mag.Zones()[0].SetName(name + "_Zone")
    for p in parent.Parameters():  # type: poser.ParmType
        if (not p.Name().strip()) or p.Name().startswith(oldname):
            p.SetName(name)

    SCENE.SelectActor(ac)
    return mag, base


def getActorList(unfiltered=False, withgeometry=False, isbodypart=False):
    def excluded(ac):
        if unfiltered:
            return False
        if withgeometry:
            if ac.Geometry() is None \
                    or ac.Geometry().NumVertices() == 0:
                return True
        if isbodypart and not ac.IsBodyPart():
            return True
        return ac.IsDeformer() or ac.IsBase() or ac.IsZone() or \
               ac.IsCamera() or ac.IsControlProp() or ac.IsLight() or \
               ac.IsMeasurement()

    return sorted([ac.Name() for ac in SCENE.Actors()
                   if not excluded(ac)],
                  key=lambda a: a[0].upper() + a[1:])


class MagnetDialog(wx.Dialog):
    def __init__(self, *args, **kwargs):
        wx.Dialog.__init__(self, *args, **kwargs)
        self.gb_manager = GridBagManager(parent=self,
                                         sizer=wx.GridBagSizer(0, 0),
                                         flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL | wx.EXPAND,
                                         border=2
                                         )
        self.SetBackgroundColour(BG_COLOR)
        self.SetForegroundColour(FG_COLOR)

        self.dialogitems = self.gb_manager.widgets

        f = wx.Font(SYS_FONT.GetPointSize() + 2, family=SYS_FONT.GetFamily(),
                    style=SYS_FONT.GetStyle(), weight=wx.BOLD)
        headline = PrepCtrl(wx.StaticText, id=-1, label="Magnet Utility", font=f)

        self.InitUI(self.gb_manager)

        main_sizer = wx.BoxSizer(wx.VERTICAL)
        main_sizer.Add(headline, 0, wx.ALL | wx.ALIGN_LEFT, 5)
        main_sizer.AddStretchSpacer(0)
        main_sizer.Add(self.gb_manager.sizer, 1, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, 5)
        btn_sizer = ButtonRow(wx.Button(self, id=wx.ID_OK),
                              wx.Button(self, id=wx.ID_CANCEL))
        main_sizer.Add(btn_sizer, 0, wx.ALL | wx.ALIGN_CENTER, 5)

        self.SetAutoLayout(True)
        self.SetSizer(main_sizer)
        main_sizer.Fit(self)

        self.Bind(wx.EVT_WINDOW_DESTROY, self.onClose)
        self.Bind(wx.EVT_SIZE, self.onSize)

    def get_item_by_name(self, *args):
        return self.gb_manager.get_item_by_name(*args)

    def onClose(self, ev):
        ev.Skip()

    def onSize(self, ev):
        ev.Skip()

    def onCheckBox(self, ev):
        ev.Skip()

    def onChoice(self, ev):
        ev.Skip()
        return

        obj = ev.GetEventObject()  # type: wx.Choice
        idx = obj.GetSelection()
        name = str(obj.GetString(idx))
        c = name[0]
        comp = "LR" if c.isupper() else "lr"
        opposite = comp[c[0].lower() == "l"] + name[1:]
        try:
            idx = getActorList().index(opposite)
        except IndexError:
            idx = None

        print("OPPOSITE", opposite if idx is not None else "None")

        obj = self.FindWindowByName("makepair")
        if obj:
            obj.SetValue(idx is not None)

    def InitUI(self, gbm):
        actorlist = getActorList(unfiltered=True)
        actoridx = actorlist.index(SCENE.CurrentActor().Name())

        # Attribute "key" is actually exchanged with "name".
        # Using "key" makes clear it is the key used in the
        # dictionary with results (after dialog is closed).
        gbm.addrows(
                Labeled(wx.Choice, label="Actor", key="actor",
                        choices=actorlist, index=actoridx,
                        tooltip="Create new magnet in this actor",
                        bind=(wx.EVT_CHOICE, self.onChoice)),
                Labeled(wx.TextCtrl, label="Name", key="Magname",
                        tooltip="New magnets name"),
                Labeled(wx.TextCtrl, label="Base Size %", key="b_size", style=wx.TE_RIGHT,
                        value="15", tooltip="Size of base control in percent"),
                Labeled(wx.TextCtrl, label="Zone Size %", key="z_size", style=wx.TE_RIGHT,
                        value="10", tooltip="Size of Zone control in percent"),
                Labeled(wx.CheckBox, label="Zone Wireframe", key="wframe", style=wx.ALIGN_RIGHT,
                        value=True, tooltip="Check to set zones displaystyle to wireframe"),
                None,  # an empty row
                Labeled(wx.CheckListBox, label="Affected Actors", key="affected",
                        choices=getActorList(withgeometry=True),
                        tooltip="Select actors affected by this deformer."),
                #                Labeled(wx.CheckBox, label="Create Pair", key="makepair", style=wx.ALIGN_RIGHT,
                #                        value=False, tooltip="Make another magnet at the other side"),

        )

        gbm.bindCtrls(wx.CheckBox, wx.EVT_CHECKBOX, self.onCheckBox)


with MagnetDialog(None, -1, style=wx.DEFAULT_DIALOG_STYLE) as dlg:
    res = dlg.ShowModal()
    if res == wx.ID_OK:
        data = dlg.gb_manager.items_as_dict()
        magname = data["Magname"]
        #print("Mag Name:", magname)
        #print("Actor:", data["actor"])
        #print("Affected:", data["affected"])

        if magname.strip() and SCENE.Actor(data["actor"]):
            ac = SCENE.Actor(data["actor"])
            SCENE.SelectActor(ac)
            mag, base = CreateMagnet(name=magname)
            if data["wframe"]:
                [z.SetDisplayStyle(poser.kDisplayCodeWIREFRAME) for z in mag.Zones()]
            v = int(data["b_size"])
            if v:
                base.SetParameter("Scale", v / 100.0)
            v = int(data["z_size"])
            if v:
                [z.SetParameter("Scale", v / 100.0) for z in mag.Zones()]

            mag.SetMagnetAffectedObjects([SCENE.Actor(objname)
                                          for objname in data.get("affected", [])] + [ac,])

            # rename Parameter names in affected actors
            for actor in mag.MagnetAffectedObjects():
                for p in actor.Parameters():
                    if p.Name().startswith(mag.Name()):
                        p.SetName(magname)
                        break
